package com.hai.order.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.hai.common.exception.ErrorCode;
import com.hai.common.exception.ErrorHelp;
import com.hai.common.exception.SysCode;
import com.hai.config.HuiLianTongUnionCardConfig;
import com.hai.config.TuanYouConfig;
import com.hai.entity.*;
import com.hai.enum_type.OilCardRecordSourceTypeEnum;
import com.hai.enum_type.OrderPayTypeEnum;
import com.hai.enum_type.OrderPushType;
import com.hai.order.service.OrderPaySuccessService;
import com.hai.order.service.OrderService;
import com.hai.order.type.OrderChildStatus;
import com.hai.order.type.OrderPayType;
import com.hai.order.type.OrderProductType;
import com.hai.order.type.OrderStatus;
import com.hai.service.HighOilCardService;
import com.hai.service.HighUserCardService;

import javax.annotation.Resource;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

/**
 * @className: OrderPayServiceImpl
 * @author: HuRui
 * @date: 2022/9/5
 **/
public class OrderPayServiceImpl implements OrderPayService {

    @Resource
    private OrderPaySuccessService orderPaySuccessService;

    @Resource
    private HighUserCardService userCardService;

    @Resource
    private HighOilCardService oilCardService;

    @Resource
    private OrderService orderService;

    @Override
    public HighOrder hltCardPay(HighOrder order, HighUserCard userCard) throws Exception {
        String goodsDesc = "";
        String tranDesc = "";
        String instCode = "11101527";
        String businessType = "hisen_consume";

        if (order.getProductType().equals(OrderProductType.PRODUCT_TYPE5.getNumber())) {
            instCode = "11101526";
            businessType = "sinopec_oil_code";
            goodsDesc = "购买加油券";
            tranDesc = order.getHighChildOrderList().get(0).getGoodsName();

        } else if (order.getProductType().equals(OrderProductType.PRODUCT_TYPE4.getNumber())) {
            goodsDesc = "积分充值" + order.getPayPrice() + "元";
            tranDesc = "嗨森逛积分充值" + order.getPayPrice() + "元";

        } else if (order.getProductType().equals(OrderProductType.PRODUCT_TYPE6.getNumber())) {
            goodsDesc = "加油站加" + order.getPayPrice() + "元油";
            tranDesc = "【" + order.getHighChildOrderList().get(0).getGoodsName() + "】加油" + order.getPayPrice() + "元";

        } else if (order.getProductType().equals(OrderProductType.PRODUCT_TYPE7.getNumber())) {
            goodsDesc = "购买优惠券包";
            tranDesc = "购买" + order.getHighChildOrderList().get(0).getGoodsName() + "优惠券包";

        } else {
            throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "暂时无法支付，请使用其他支付方式");
        }

        // 工会卡支付
        JSONObject consumption = HuiLianTongUnionCardConfig.consumption(order.getOrderNo(), userCard.getCardNo(), order.getPayPrice(), businessType, instCode, goodsDesc, tranDesc);
        if (!consumption.getString("respCode").equals("0000")) {
            throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, consumption.getString("respMessage"));
        }
        // 响应参数
        JSONObject consumptionResult = HuiLianTongUnionCardConfig.resolveResponse(consumption.getString("data"));
        if (consumptionResult.getBoolean("success") != true) {
            throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, consumptionResult.getString("message"));
        }
        order.setMemCardId(userCard.getId());
        order.setMemCardNo(userCard.getCardNo());
        order.setMemCardType(userCard.getType());
        order.setPaySerialNo(consumptionResult.getString("orderId")); // 支付流水号
        order.setPayTime(new Date());
        order.setPayType(OrderPayType.PAY_TYPE4.getNumber());
        order.setPayRealPrice(order.getPayPrice());
        order.setOrderStatus(OrderStatus.STATUS2.getNumber());

        for (HighChildOrder childOrder : order.getHighChildOrderList()) {
            childOrder.setChildOrderStatus(OrderChildStatus.STATUS2.getNumber());
            childOrder.setPayTime(new Date());
        }
        HighOrder updateOrderData = orderService.updateOrderData(order);

        // 异步处理订单业务
        orderPaySuccessService.asyncOrderPaySuccess(order);

        return updateOrderData;

    }

    @Override
    public HighOrder oilCardPay(HighOrder order, HighUserCard userCard) {
        // 油卡扣款
        Map<String, Object> consumeMap = new HashMap<>();
        consumeMap.put("sourceType", OilCardRecordSourceTypeEnum.type2.getType());
        consumeMap.put("sourceId", order.getId());
        consumeMap.put("sourceOrderNo", order.getOrderNo());
        consumeMap.put("sourceContent", "订单号：" + order.getOrderNo() + ",消费：￥" + order.getPayPrice());
        oilCardService.consume(userCard.getCardNo(), order.getPayPrice(), consumeMap);

        order.setMemCardType(userCard.getType());
        order.setMemCardId(userCard.getId());
        order.setMemCardNo(userCard.getCardNo());
        order.setPayTime(new Date());
        order.setPayType(OrderPayTypeEnum.type7.getType());
        order.setPayRealPrice(order.getPayPrice());
        order.setOrderStatus(OrderStatus.STATUS2.getNumber());
        HighOrder updateOrderData = orderService.updateOrderData(order);

        // 异步处理业务
        orderPaySuccessService.asyncOrderPaySuccess(order);
        return updateOrderData;
    }
}
